﻿# -*- coding: UTF-8 -*-
#!/usr/local/bin/python

#Copyright (C) 2007 by raptor
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# 根据公历时间换算四柱（生辰八字）
# author : raptor@we8log.com
# spec. thanks : http://hi.baidu.com/doloveselin/blog/item/dbfe2a1f07aee7caa6866973.html
#  公元农历 from http://www.firefox.net.cn/
# create : 2007/10/17
# revision : 2007/11/08 -- 增加星座功能
#            2007/12/28 -- 修正一个计算错误之处，并调整部分数据，以使最近几年的时间更准确

from datetime import datetime, timedelta
import sys

class FourBar :
    Gan  = ["甲","乙","丙","丁","戊","己","庚","辛","壬","癸"]
    Zhi  = ["子","丑","寅","卯","辰","巳","午","未","申","酉","戌","亥"]
    Term = ["小寒","大寒","立春","雨水","惊蛰","春分","清明","谷雨","立夏","小满","芒种","夏至","小暑","大暑","立秋","处暑","白露","秋分","寒露","霜降","立冬","小雪","大雪","冬至"]
    TermInfo = [0,21208,42467,63836,85337,107014,128867,150921,173149,195551,218072,240693,263343,285989,308563,331033,353350,375494,397447,419210,440795,462224,483532,504758]
    Constellations = ["摩羯座", "水瓶座", "双鱼座", "白羊座", "金牛座", "双子座", "巨蟹座", "狮子座", "处女座", "天平座", "天蝎座", "射手座"]

    def __init__(self, s=''):
        if len(s)<8 :
            self.borntime=datetime(1900,1,1).today()
        else :
            year=int(s[0:4])
            month=int(s[4:6])
            day=int(s[6:8])
            hour=0
            minute=0
            if len(s)>=10 :
                hour=int(s[8:10])
                if len(s)>=12:
                    minute=int(s[10:12])
            self.borntime=datetime(year,month,day,hour,minute)

    # month = 0-11
    # index = 0-1
    def getTermDay(self,year,month,index):
        # 31556925974.7为以ms为单位的节气年周期
        tdyear=timedelta(365,20925,974700)
        md=self.TermInfo[month*2+index]
        tdterm=timedelta(md/1440,(md%1440)*60)
        # 1900年小寒时间：1月6日2时5分
        # 1900年小寒时间调整为：1月6日3时30分，牺牲早年的时间精度，以使最近几年的节气时间更准确
        offDate=datetime(1900,1,6,3,30)+tdyear*(year-1900)+tdterm
        return offDate

    def getLichun(self,year):
        return self.getTermDay(year,1,0) # 立春时刻

    def getYearNum(self,dt):
        year_start=self.getLichun(dt.year)
        year=dt.year
        if dt<year_start :
            year-=1
        return year-1900+36

    def cyclical(self,num) :
        return unicode(self.Gan[num%10]+self.Zhi[num%12],'utf-8')

    def getYearBar(self):
        return self.cyclical(self.getYearNum(self.borntime))

    def getMonthBar(self):
        month_start=self.getTermDay(self.borntime.year,self.borntime.month-1,0)
        month_index=self.borntime.month
        if self.borntime<month_start :
            month_index-=1
        year_gan=self.getYearNum(self.borntime)%10
        month_gan=month_index-2+(year_gan+1)%5*2
        if self.borntime<self.getLichun(self.borntime.year) :
            month_gan+=2
        return unicode(self.Gan[month_gan%10]+self.Zhi[month_index%12],'utf-8')

    def getDayNum(self,dt):
        # 1900年2月20日为甲子日，换日时间为23:00
        day_diff=dt-datetime(1900,2,19,23)
        return day_diff.days

    def getDayBar(self):
        return self.cyclical(self.getDayNum(self.borntime))

    def getHourBar(self):
        hour_zhi=((self.borntime.hour+1)/2)%12
        day_gan=self.getDayNum(self.borntime)%10
        hour_gan=hour_zhi+(day_gan%5)*2
        return unicode(self.Gan[hour_gan%10]+self.Zhi[hour_zhi],'utf-8')

    def getConstellation(self):
        month_start=self.getTermDay(self.borntime.year,self.borntime.month-1,1)
        month_index=self.borntime.month
        if self.borntime<month_start :
            month_index-=1
        return unicode(self.Constellations[month_index%12],'utf-8')

if __name__ == '__main__' :
    if len(sys.argv)>1 :
        fb=FourBar(sys.argv[1])
        print fb.getYearBar(), fb.getMonthBar(), fb.getDayBar(), fb.getHourBar(), fb.getConstellation()
    else :
        print """
Usage : fourbar.py yyyymmdd[hhnn]
"""
