﻿# -*- coding: UTF-8 -*-
#!/usr/local/bin/python

from datetime import datetime, date
import unittest
from fourbar import FourBar

class TestMain( unittest.TestCase ) :
	def setUp(self) :
		self.fb=FourBar()

	def testFbclass(self) :
		tfb=FourBar('200710181409')
		self.assertEqual(tfb.borntime, datetime(2007,10,18,14,9))

	def testTermDay(self) :
		dt=self.fb.getTermDay(2007,1,0)
		self.assertEqual(dt.date(), date(2007,2,4))

	def testYearBar(self) :
		self.fb.borntime=datetime(2007,2,4,13,14)
		self.assertEqual(self.fb.getYearBar(), u'丙戌')
		self.fb.borntime=datetime(2007,2,4,13,15)
		self.assertEqual(self.fb.getYearBar(), u'丁亥')

	def testMonthBar(self) :
		self.fb.borntime=datetime(2006,1,4)
		self.assertEqual(self.fb.getMonthBar(), u'戊子')
		self.fb.borntime=datetime(2006,1,6)
		self.assertEqual(self.fb.getMonthBar(), u'己丑')
		self.fb.borntime=datetime(2007,2,4,13,14)
		self.assertEqual(self.fb.getMonthBar(), u'辛丑')
		self.fb.borntime=datetime(2007,2,4,13,15)
		self.assertEqual(self.fb.getMonthBar(), u'壬寅')
		self.fb.borntime=datetime(2007,10,7)
		self.assertEqual(self.fb.getMonthBar(), u'己酉')
		self.fb.borntime=datetime(2007,10,10)
		self.assertEqual(self.fb.getMonthBar(), u'庚戌')
		self.fb.borntime=datetime(2007,12,6)
		self.assertEqual(self.fb.getMonthBar(), u'辛亥')
		self.fb.borntime=datetime(2007,12,8)
		self.assertEqual(self.fb.getMonthBar(), u'壬子')

	def testDayBar(self) :
		self.fb.borntime=datetime(1900,2,19,22,59)
		self.assertEqual(self.fb.getDayBar(), u'癸亥')
		self.fb.borntime=datetime(1900,2,19,23,1)
		self.assertEqual(self.fb.getDayBar(), u'甲子')
		self.fb.borntime=datetime(1900,2,20)
		self.assertEqual(self.fb.getDayBar(), u'甲子')

	def testHourBar(self) :
		self.fb.borntime=datetime(1900,2,19,22,59)
		self.assertEqual(self.fb.getHourBar(), u'癸亥')
		self.fb.borntime=datetime(1900,2,19,23,1)
		self.assertEqual(self.fb.getHourBar(), u'甲子')
		self.fb.borntime=datetime(1900,2,20,1,1)
		self.assertEqual(self.fb.getHourBar(), u'乙丑')
		self.fb.borntime=datetime(1900,2,21,1,1)
		self.assertEqual(self.fb.getHourBar(), u'丁丑')

	def testConstellation(self) :
		self.fb.borntime=datetime(2007,2,19)
		self.assertEqual(self.fb.getConstellation(), u'水瓶座')
		self.fb.borntime=datetime(2007,2,20)
		self.assertEqual(self.fb.getConstellation(), u'双鱼座')

if __name__ == '__main__' :
	unittest.main()
